/*
* Copyright 2001-2015 VIA Technologies, Inc. All Rights Reserved.
* Copyright 2001-2012 S3 Graphics, Inc. All Rights Reserved.
*
* Permission is hereby granted, free of charge, to any person obtaining a
* copy of this software and associated documentation files (the "Software"),
* to deal in the Software without restriction, including without limitation
* the rights to use, copy, modify, merge, publish, distribute, sub license,
* and/or sell copies of the Software, and to permit persons to whom the
* Software is furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice (including the
* next paragraph) shall be included in all copies or substantial portions
* of the Software.
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL
* THE AUTHOR(S) OR COPYRIGHT HOLDER(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
* DEALINGS IN THE SOFTWARE.
*/

#include "drmP.h"
#include <linux/console.h>
#include <linux/module.h>

#if ! defined(LINUX_VERSION_CODE)  
#include <linux/version.h>
#endif

#define DRIVER_AUTHOR   "S3G"

#define DRIVER_NAME     "s3g_drm"
#define DRIVER_DESC     "S3G DRM Pro"
#define DRIVER_DATE     "20131231"

#define DRIVER_MAJOR        1
#define DRIVER_MINOR        1
#define DRIVER_PATCHLEVEL   0

#if (LINUX_VERSION_CODE < 0x40800)
int  via_chrome9_drm_resume(struct drm_device *dev)
{
	return 0;
}

int  via_chrome9_drm_suspend(struct drm_device *dev, pm_message_t state)
{
	return 0;
}
#endif
#if (LINUX_VERSION_CODE < 0x40800)
DEFINE_PCI_DEVICE_TABLE(pciidlist) = {
#else
static struct pci_device_id pciidlist[] = {
#endif
	{0x1106, 0x3A01, PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0}, 
	{0, 0, 0}
};


#if (LINUX_VERSION_CODE >= 0x030000)
static struct pci_driver pcidriver = {
		 .name = DRIVER_NAME,
		 .id_table = pciidlist,
};
#endif

struct file_operations s3g_drm_fops = {
    .owner = THIS_MODULE,
    .open = drm_open,
    .release = drm_release,
    .unlocked_ioctl = drm_ioctl,
    .mmap = NULL,
    .poll = drm_poll,
#ifdef HAS_drm_fasync
    .fasync = drm_fasync,
#endif
    .read = drm_read,
    .llseek = noop_llseek,
};
static struct drm_driver driver = {
	.driver_features =
		DRIVER_HAVE_DMA | DRIVER_GEM |
#ifdef HAS_DRIVER_FB_DMA
		DRIVER_FB_DMA |
#endif
#ifdef HAS_DRIVER_USE_MTRR
		DRIVER_USE_MTRR |
#endif
		DRIVER_HAVE_IRQ | DRIVER_IRQ_SHARED,
	.load = NULL,
	.preclose = NULL,
	.lastclose = NULL,
	.unload = NULL,
#if (LINUX_VERSION_CODE < 0x40800)
	.suspend = via_chrome9_drm_suspend,
	.resume = via_chrome9_drm_resume,
#endif
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(3,18,0))
#if (LINUX_VERSION_CODE <= KERNEL_VERSION(4,12,0))
    .set_busid = drm_pci_set_busid,
#endif
#endif
	.irq_preinstall = NULL,
	.irq_postinstall = NULL,
	.irq_uninstall = NULL,
	.irq_handler = NULL,
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4,10,0))
	.get_vblank_counter = NULL,
#else
	.get_vblank_counter = drm_vblank_count,
#endif
	.enable_vblank = NULL,
	.disable_vblank = NULL,
#if (LINUX_VERSION_CODE <= KERNEL_VERSION(3,5,35))
	.reclaim_buffers = drm_core_reclaim_buffers,
#endif
#ifdef HAS_gem_init_object
	.gem_init_object = NULL,
#endif
	.gem_free_object = NULL,
	.ioctls = NULL,
#if (LINUX_VERSION_CODE < KERNEL_VERSION(3,2,50))
	.fops = {
		 .owner = THIS_MODULE,
		 .open = drm_open,
		 .release = drm_release,
		 .unlocked_ioctl = drm_ioctl,
		 .mmap = NULL,
		 .poll = drm_poll,
		 .fasync = drm_fasync,
		 .read = drm_read,
		 .llseek = noop_llseek,
	},
#else
    .fops = &s3g_drm_fops,
#endif
#if !(LINUX_VERSION_CODE >= 0x030000)
	.pci_driver = {
		 .name = DRIVER_NAME,
		 .id_table = pciidlist,
	},
#endif

	.name = DRIVER_NAME,
	.desc = DRIVER_DESC,
	.date = DRIVER_DATE,
	.major = DRIVER_MAJOR,
	.minor = DRIVER_MINOR,
	.patchlevel = DRIVER_PATCHLEVEL,
};
static int __init s3g_drm_init(void)
{
	driver.num_ioctls = 0;
#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4,13,0))
	return pci_register_driver(&pcidriver);
#else
#if (LINUX_VERSION_CODE >= 0x030000)
	return drm_pci_init(&driver, &pcidriver);
#else
	return drm_init(&driver);
#endif
#endif
}

static void __exit s3g_drm_exit(void)
{

#if (LINUX_VERSION_CODE >= KERNEL_VERSION(4,13,0))
	return pci_unregister_driver(&pcidriver);
#else
#if (LINUX_VERSION_CODE >= 0x030000)
	return drm_pci_exit(&driver, &pcidriver);
#else
	drm_exit(&driver);
#endif
#endif
}

module_init(s3g_drm_init);
module_exit(s3g_drm_exit);

MODULE_AUTHOR(DRIVER_AUTHOR);
MODULE_DESCRIPTION(DRIVER_DESC);
MODULE_LICENSE("GPL and additional rights");
MODULE_DEVICE_TABLE(pci, pciidlist);
